#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "host_file_accessor.h"

HostFileAccessor::HostFileAccessor(const char *path, fileid_t fileid)
: Accessor(fileid), _path(path)
{
    /* Initialise the size of the file. */

    struct stat buf;

    if (!stat(_path, &buf))
        _size = buf.st_size;
    else
        _size = 0;
}

/* Perform any closing operation on the file. */

void HostFileAccessor::close()
{
    fclose(_fp);
}

/* Perform any opening operation on the file. */

void HostFileAccessor::open()
{
    _fp = fopen(_path, "r");
}

/* Data transfer helper methods. */

void HostFileAccessor::fill_populated(Flexpage *flexpage)
{
    offset_t filepos = flexpage->base_offset;
    offset_t addr = flexpage->base_addr;

    /* Seek to the offset in the file. */

    fseek(_fp, filepos, SEEK_SET);

    /* Tag the region with file state. */

    flexpage->region->fill(fileid, filepos);

    /* Fill the region with file content. */

    size_t nread = fread((void *) addr, sizeof(char), flexpage->size, _fp);

    /* Pad the flexpage with zero. */

    memset((void *) (addr + nread), 0, flexpage->size - nread);
}

void HostFileAccessor::flush_populated(Flexpage *flexpage)
{
    flexpage->region->flush();
}

// vim: tabstop=4 expandtab shiftwidth=4
